/*
 * Bogdan Grama
 * Decodor coduri morse
 * 332 CC
 */

#define F_CPU 16000000L

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <util/delay.h>
#include "lcd.h"

#define DOT 20
#define LINE 40
#define UNKNOWN 12

#define LED_PORT PORTB
#define LED1 PB0
#define LED2 PB1
#define LED3 PB2
#define RESET '*'
#define SPACE ' '

#define STATE_PRESSED 1
#define STATE_FREE 2
#define STATE_DECODE 3
#define MAX_LENGTH 100

#define SDOT '.'
#define SLINE '-'
#define WI 1000

volatile unsigned long count;

void enable_button(void) {
  /*
   * Functie care initializeaza butonul 
   * pe portul D, acelasi ca si pe placa.
   */
    DDRD &= ~(1 << PD6);
    PORTD |= (1 << PD6);
}

int is_button_pressed(void) {
  /*
   * Functie care verifica daca e apasat 
   * un buton.
   */
    return (PIND & (1 << PD6)) == 0;
}

void enable_timer(void) {
  /*
   * Activare intrerupere.
   */
    TCCR1B = _BV(WGM12) | _BV(CS11);
    OCR1A = 0x8235;

    TIMSK |= _BV(OCIE1A);

    sei();

    count = 0;
}

ISR(TIMER1_COMPA_vect)
{
  /*
   * Actiune intrerupere 
   */
    count++;
}

char *LITERE[26] = {
  /*
   * Aici se retine fiecarui secvente 
   * ce Litera ii corespunde.
   */
    ".-",	//A 
    "-...",     //B 
    "-.-.",	//C
    "-..",	//D
    ".",	//E
    "..-.",	//F
    "--.",	//G
    "....",	//H
    "..",	//I
    ".---",	//J
    "-.-",	//K
    ".-..",	//L
    "--",	//M
    "-.",	//N
    "---",	//O
    ".--.",	//P
    "--.-",	//Q
    ".-.",	//R
    "...",	//S
    "-",	//T
    "..-",	//U
    "...-",	//V
    ".--",	//W
    "-..-",	//X
    "-.--",	//Y
    "--.."      //Z
};

char *CIFRE[10] = {
  /*
   * Corespondenta cu cifre
   */
    "-----",    //0
    ".----",    //1
    "..---",	//2
    "...--",	//3
    "....-",	//4
    ".....",	//5
    "-....",	//6
    "--...",	//7
    "---..",	//8
    "----.",    //9
};

char humanize(char *morse_buffer)
{
  /*
   * Functie care transporma secventa de caractere 
   * reprezentata de codul morse in litera/cifra.
   * Codul de reset este: .---.
   * Codul de spatiu este: --..--
   * Am cautat niste caractere care nu existau si
   * astea au fost cele mai inspirate pe care le-am
   * putut gasi.
   */
    int i;
    for ( i = 0 ; i < 26; i++)
        if ( !strcmp(LITERE[i], morse_buffer))
            return i + 'A';

    for ( i = 0 ; i < 10; i++)
        if ( !strcmp(CIFRE[i], morse_buffer))
            return i + '0';

    if ( !strcmp(".---.",morse_buffer) )
        return RESET;
    
    if ( !strcmp("--..--",morse_buffer) )
        return SPACE;
    return UNKNOWN;
}

int main(void)
{

    LCD_init();
    _delay_ms(100);

/*//    afisare mesaj welcome
    LCD_waitNotBusy();
    LCD_writeInstruction(LCD_INSTR_clearDisplay);
    _delay_ms(WI);
    LCD_print2("*Bine ati venit*","****************");
    _delay_ms(10000);
    LCD_print2("//Bogdan Grama\\\\", "-----332 CC-----");
    _delay_ms(10000);
    LCD_writeInstruction(LCD_INSTR_clearDisplay);
    _delay_ms(WI);*/
    
    enable_button();
    enable_timer();

    int state = STATE_FREE, switched = 0, i;


    char morse_buffer[MAX_LENGTH];
    int morse_idx = 0;
    morse_buffer[0] = 0;

    char word_buffer[MAX_LENGTH];
    int word_idx = 0;
    word_buffer[0] = 0;

    char symbol = ' ';
    char letter;
    char buf[10] ="";

    int toClear = 0;
    while (1) {

        _delay_ms(20);
        
        int pressed = is_button_pressed();
        if (pressed) {
            if (STATE_DECODE == state) {
                switched = 0;
            }
            if ( !switched )
                count = 0;
            switched = 1;
            state = STATE_PRESSED;
            if ( count > 0 ) {
                symbol = SDOT;
            }
            if (count >= DOT ) {
                symbol = SLINE;
            }
        } else {
            if ( STATE_DECODE == state) {
                if ( switched && count > 3*LINE) {
                    for ( i = 0; i < 7 ; i ++ ) {
                        
                        _delay_ms(30);
                        _delay_ms(30);
                    }
                    morse_buffer[morse_idx] = 0;
                    letter = humanize(morse_buffer);
                    morse_buffer[0] = 0;
                    morse_idx = 0;
                    LCD_writeInstruction(LCD_INSTR_nextLine);
		    _delay_ms(10);
		    LCD_print("               ");
		    _delay_ms(10);
		    LCD_writeInstruction(LCD_INSTR_returnHome);
		    _delay_ms(10);
		    LCD_writeInstruction(LCD_INSTR_nextLine);
                    LCD_print("caracter:");
                    if (letter == UNKNOWN) {
                        sprintf(buf,"ERROR");
                    } else if (letter == RESET) {
                        sprintf(buf,"RESET");
                    } else {
                        sprintf(buf,"%c",letter);
                        word_buffer[word_idx] = letter;
                        word_idx++;
                        word_buffer[word_idx] = 0;
                    }

                    LCD_print(buf);
                    toClear = 1;
                    _delay_ms(1200);
		    if (letter == RESET){
		      word_buffer[0] = morse_buffer[0] = 0;
		      word_idx = morse_idx = 0;
		    }
                    LCD_writeInstruction(LCD_INSTR_clearDisplay);
                    LCD_print2(morse_buffer,word_buffer);
                    switched = 0;
                }
                if ( !switched ) {
                    count = 0;
                }
            } else {
                state = STATE_DECODE;
                count = 0;
                if ( switched) {
                    morse_buffer[morse_idx] = symbol;
                    morse_idx++;
                }
                morse_buffer[morse_idx] = 0;
                LCD_writeInstruction(LCD_INSTR_clearDisplay);
                sprintf(buf,"%c",symbol);
                LCD_print2(morse_buffer,word_buffer);
            }
        }
    }

}
